@@ -89,6 +89,8 @@ class Processor:
 
 
         self.conda_prefix = args.conda_prefix
+        self.tmpdir = args.tmpdir
+        self.resources = args.resources
         self.output = args.output
         self.threads = args.max_threads
         self.max_memory = args.max_memory
@@ -105,10 +107,22 @@ class Processor:
             self.min_contig_size = args.min_contig_size
             self.min_bin_size = args.min_bin_size
             self.semibin_model = args.semibin_model
-            self.skip_binners = [binner.lower() for binner in args.skip_binners]
-            self.check_binners_to_skip()
-            if "get_bam_indices" not in self.workflows and "complete_assembly" not in self.workflows:
-                self.workflows.insert(0, 'get_bam_indices')
+
+            if args.skip_binners is not None:
+                # skipping binners is tricky and requires that bam indices are produced
+                # so we explicitly place the bam indices rule in the DAG before any binning occurs
+                self.skip_binners = [binner.lower() for binner in args.skip_binners]
+                self.check_binners_to_skip()
+                if not any(rule in self.workflows for rule in ["get_bam_indices", "complete_assembly", "complete_assembly_with_qc"]):
+                    self.workflows.insert(0, 'get_bam_indices')
+                elif any(rule in self.workflows for rule in ["complete_assembly", "complete_assembly_with_qc"]):
+                    indices = [idx for idx, rule in enumerate(self.workflows) if rule in ["complete_assembly", "complete_assembly_with_qc"]]
+                    if len(indices) > 1:
+                        logging.critical(f"Rules 'complete_assembly' and 'complete_assembly_with_qc' both found in DAG or one found multiple times.")
+                        logging.critical(f"Please revise your --workflow parameter.")
+                        sys.exit(1)
+                    self.workflows.insert(indices[0] + 1, 'get_bam_indices')
+
         except AttributeError:
             self.min_contig_size = 1500
             self.min_bin_size = 200000
@@ -149,6 +163,7 @@ class Processor:
 
         try:
             self.short_percent_identity = args.short_percent_identity
+
             if args.coupled != "none":
                 self.pe1 = args.coupled[::2]
                 self.pe2 = args.coupled[1::2]
@@ -166,10 +181,15 @@ class Processor:
             self.pe2 = 'none'
             self.short_percent_identity = 'none'
 
+
         try:
             self.kmer_sizes = args.kmer_sizes
+            self.use_megahit = args.use_megahit
+            self.coassemble = args.coassemble
         except AttributeError:
             self.kmer_sizes = ['auto']
+            self.use_megahit = False
+            self.coassemble = True
 
         try:
             self.mag_directory = os.path.abspath(args.directory) if args.directory is not None else 'none'
@@ -285,17 +305,17 @@ class Processor:
         conf["long_reads"] = self.longreads
         conf["long_read_type"] = self.longread_type
         conf["kmer_sizes"] = self.kmer_sizes
+        conf["use_megahit"] = self.use_megahit
+        conf["coassemble"] = self.coassemble
         conf["min_contig_size"] = int(self.min_contig_size)
         conf["min_bin_size"] = int(self.min_bin_size)
         conf["gtdbtk_folder"] = self.gtdbtk
         conf["eggnog_folder"] = self.eggnog
         conf["strain_analysis"] = self.strain_analysis
-
         conf["checkm2_db_folder"] = self.checkm2_db
         conf["use_checkm2_scores"] = self.use_checkm2_scores
         conf["mag_directory"] = self.mag_directory
         conf["mag_extension"] = self.mag_extension
-        # conf["mags"] = self.mags
         conf["previous_runs"] = self.previous_runs
         conf["min_completeness"] = self.min_completeness
         conf["max_contamination"] = self.max_contamination
@@ -303,6 +323,7 @@ class Processor:
         conf["precluster_ani"] = self.precluster_ani
         conf["precluster_method"] = self.precluster_method
         conf["pggb_params"] = self.pggb_params
+        conf["tmpdir"] = self.tmpdir
 
         with open(self.config, "w") as f:
             yaml.dump(conf, f)
@@ -406,7 +427,7 @@ class Processor:
                 "snakemake --snakefile {snakefile} --directory {working_dir} "
                 "{jobs} --rerun-incomplete "
                 "--configfile '{config_file}' --nolock "
-                "{profile} {conda_frontend} --use-conda {conda_prefix} "
+                "{profile} {conda_frontend} {resources} --use-conda {conda_prefix} "
                 "{dryrun}{notemp}{args}"
                 " {target_rule}"
             ).format(
@@ -420,7 +441,8 @@ class Processor:
                 args=snakemake_args,
                 target_rule=workflow if workflow != "None" else "",
                 conda_prefix="--conda-prefix " + self.conda_prefix,
-                conda_frontend="--conda-frontend " + conda_frontend
+                conda_frontend="--conda-frontend " + conda_frontend,
+                resources=f"--default-resources \"tmpdir='{self.tmpdir}'\" --resources mem_mb={int(self.max_memory)*1024} {self.resources}" if not dryrun else ""
             )
             logging.info("Executing: %s" % cmd)
             try:
@@ -430,6 +452,100 @@ class Processor:
                 logging.critical(e)
                 exit(1)
 
+
+def process_batch(args, prefix):
+    '''
+    Function for handling and processing aviary commands in batches
+    The user supplies a tab separated batch file with six defined columns. Aviary is run on each line
+    of the batch file using the specified workflow.
+
+    If the user wishes, the results are then clustered.
+    '''
+    import pandas as pd
+
+    logging.info(f"Reading batch file: {args.batch_file}")
+    batch = pd.read_csv(args.batch_file, sep=None, engine='python')
+    if len(batch.columns) != 7:
+        logging.critical(f"Batch file contains incorrect number of columns ({len(batch.columns)}). Should contain 7.")
+        logging.critical(f"Current columns: {batch.columns}")
+        sys.exit()
+
+    if args.build:
+        try:
+            args.cmds = args.cmds + '--conda-create-envs-only '
+        except TypeError:
+            args.cmds = '--conda-create-envs-only '
+
+    if args.use_unicycler:
+        args.workflow.insert(0, "combine_assemblies")
+
+    runs = []
+    args.interleaved = "none" # hacky solution to skip attribute error
+    args.coupled = "none"
+    for i in range(batch.shape[0]):
+        # process the batch line
+        sample = check_batch_input(batch.iloc[i, 0], f"sample_{i}", split=False)
+        logging.info(f"Processing {sample}")
+        s1 = check_batch_input(batch.iloc[i, 1], "none", split=True)
+        s2 = check_batch_input(batch.iloc[i, 2], "none", split=True)
+        l = check_batch_input(batch.iloc[i, 3], "none", split=True)
+        l_type = check_batch_input(batch.iloc[i, 4], "ont", split=False)
+        assembly = check_batch_input(batch.iloc[i, 5], None, split=False)
+        coassemble = check_batch_input(batch.iloc[i, 6], False, split=False)
+
+        # update the value of args
+        args.output = f"{prefix}/{sample}"
+        runs.append(args.output)
+        args.pe1 = s1
+        args.pe2 = s2
+
+        args.longreads = l
+        args.longread_type = l_type
+        args.assembly = assembly
+        args.coassemble = coassemble
+
+        # ensure output folder exists
+        if not os.path.exists(args.output):
+            os.makedirs(args.output)
+
+        # setup processor for this line
+        processor = Processor(args)
+        processor.make_config()
+
+        processor.run_workflow(cores=int(args.n_cores),
+                               dryrun=args.dryrun,
+                               clean=args.clean,
+                               conda_frontend=args.conda_frontend,
+                               snakemake_args=args.cmds)
+
+    if args.cluster:
+        logging.info(f"Beginning clustering of {len(runs)} previous Aviary runs...")
+        args.previous_runs = runs
+        args.workflow = ['complete_cluster']
+        args.output = f"{prefix}/aviary_cluster"
+        processor = Processor(args)
+        processor.make_config()
+
+        processor.run_workflow(cores=int(args.n_cores),
+                               dryrun=args.dryrun,
+                               clean=args.clean,
+                               conda_frontend=args.conda_frontend,
+                               snakemake_args=args.cmds)
+
+def check_batch_input(val, default=None, split=False, split_val=','):
+    """
+    Takes a batch entry from within a line and ensures the output makes sense for Aviary
+    Split is required for cells that separated by the split_val
+    """
+    if not isinstance(val, str):
+        return default
+
+    new_val = val.strip()
+    if split:
+        new_val = new_val.split(split_val)
+
+    return new_val
+
 def fraction_to_percent(val):
     if val <= 1:
         return val * 100
